@@ -13,10 +13,14 @@ REES = [
     "IGUACU",
     "PRNPANEMA",
 ]
+MAX_PATAMARES = 5
 MAX_SEMANAS_PRE = 10
 MAX_ESTAGIOS = 10
 MAX_SUBSISTEMAS = 10
+MAX_UTES = 300
+MAX_UHES = 300
 MAX_REES = 20
+MAX_CENARIOS = 999
 MESES = [
     "JANEIRO",
     "FEVEREIRO",
