@@ -1,19 +1,82 @@
-from idecomp.decomp.modelos.dadger import LeituraDadger
-from idecomp.decomp.modelos.dadger import TE, SB, UH, CT, UE, DP, CD, PQ  # noqa
-from idecomp.decomp.modelos.dadger import RI, IA, TX, GP, NI, DT, MP, MT  # noqa
-from idecomp.decomp.modelos.dadger import FD, VE, RE, LU, FU, FT, FI, VI  # noqa
-from idecomp.decomp.modelos.dadger import AC, IR, CI, CE, FC, TI, RQ, EZ  # noqa
-from idecomp.decomp.modelos.dadger import HV, LV, CV, HQ, LQ, CQ, AR, EV  # noqa
-from idecomp.decomp.modelos.dadger import FJ, HE, CM, RT, FP  # noqa
-from idecomp._utils.arquivo import ArquivoRegistros
-from idecomp._utils.dadosarquivo import DadosArquivoRegistros
-from idecomp._utils.escritaregistros import EscritaRegistros
-
-from copy import deepcopy
-from typing import Type, List, Optional, TypeVar, Any
+from idecomp.decomp.modelos.dadger import (
+    TE,
+    SB,
+    UH,
+    CT,
+    UE,
+    DP,
+    CD,
+    FP,
+    RI,
+    IA,
+    TX,
+    GP,
+    NI,
+    DT,
+    MP,
+    MT,
+    FD,
+    VE,
+    RE,
+    LU,
+    FU,
+    FT,
+    FI,
+    VI,
+    IR,
+    CI,
+    CE,
+    FC,
+    TI,
+    RQ,
+    EZ,
+    RT,
+    HV,
+    LV,
+    CV,
+    HQ,
+    LQ,
+    CQ,
+    AR,
+    EV,
+    FJ,
+    HE,
+    CM,
+    ACNUMPOS,
+    ACNUMJUS,
+    ACDESVIO,
+    ACVOLMIN,
+    ACVOLMAX,
+    ACCOTVOL,
+    ACCOTARE,
+    ACPROESP,
+    ACPERHID,
+    ACNCHAVE,
+    ACCOTVAZ,
+    ACCOFEVA,
+    ACNUMCON,
+    ACNUMMAQ,
+    ACPOTEFE,
+    ACALTEFE,
+    ACVAZEFE,
+    ACJUSMED,
+    ACVERTJU,
+    ACVAZMIN,
+    ACTIPERH,
+    ACJUSENA,
+    ACVSVERT,
+    ACVMDESV,
+    ACNPOSNW,
+)
 
 
-class Dadger(ArquivoRegistros):
+from cfinterface.files.registerfile import RegisterFile
+from cfinterface.components.state import ComponentState
+from cfinterface.components.register import Register
+from typing import Type, List, Optional, TypeVar, Any, Union
+
+
+class Dadger(RegisterFile):
     """
     Armazena os dados de entrada gerais do DECOMP.
 
@@ -33,81 +96,199 @@ class Dadger(ArquivoRegistros):
 
     T = TypeVar("T")
 
-    def __init__(self, dados: DadosArquivoRegistros) -> None:
-        """
-        Construtor padrão
-        """
-        super().__init__(dados)
+    AC = Union[
+        ACNUMPOS,
+        ACNUMJUS,
+        ACDESVIO,
+        ACVOLMIN,
+        ACVOLMAX,
+        ACCOTVOL,
+        ACCOTARE,
+        ACPROESP,
+        ACPERHID,
+        ACNCHAVE,
+        ACCOTVAZ,
+        ACCOFEVA,
+        ACNUMCON,
+        ACNUMMAQ,
+        ACPOTEFE,
+        ACALTEFE,
+        ACVAZEFE,
+        ACJUSMED,
+        ACVERTJU,
+        ACVAZMIN,
+        ACTIPERH,
+        ACJUSENA,
+        ACVSVERT,
+        ACVMDESV,
+        ACNPOSNW,
+    ]
+
+    REGISTERS = [
+        TE,
+        SB,
+        UH,
+        CT,
+        UE,
+        DP,
+        CD,
+        FP,
+        RI,
+        IA,
+        TX,
+        GP,
+        NI,
+        DT,
+        MP,
+        MT,
+        FD,
+        VE,
+        RE,
+        LU,
+        FU,
+        FT,
+        FI,
+        VI,
+        IR,
+        CI,
+        CE,
+        FC,
+        TI,
+        RQ,
+        EZ,
+        RT,
+        HV,
+        LV,
+        CV,
+        HQ,
+        LQ,
+        CQ,
+        AR,
+        EV,
+        FJ,
+        HE,
+        CM,
+        ACNUMPOS,
+        ACNUMJUS,
+        ACDESVIO,
+        ACVOLMIN,
+        ACVOLMAX,
+        ACCOTVOL,
+        ACCOTARE,
+        ACPROESP,
+        ACPERHID,
+        ACNCHAVE,
+        ACCOTVAZ,
+        ACCOFEVA,
+        ACNUMCON,
+        ACNUMMAQ,
+        ACPOTEFE,
+        ACALTEFE,
+        ACVAZEFE,
+        ACJUSMED,
+        ACVERTJU,
+        ACVAZMIN,
+        ACTIPERH,
+        ACJUSENA,
+        ACVSVERT,
+        ACVMDESV,
+        ACNPOSNW,
+    ]
+
+    def __init__(self, data=...) -> None:
+        super().__init__(data)
 
-    # Override
     @classmethod
     def le_arquivo(cls, diretorio: str, nome_arquivo="dadger.rv0") -> "Dadger":
-        """
-        Realiza a leitura de um arquivo "dadger.rvx" existente em
-        um diretório.
-
-        :param diretorio: O caminho relativo ou completo para o diretório
-            onde se encontra o arquivo
-        :type diretorio: str
-        :param nome_arquivo: Nome do arquivo a ser lido, potencialmente
-            especificando a revisão. Tem como valor default "dadger.rv0"
-        :type nome_arquivo: str, optional
-        :return: Um objeto :class:`Dadger` com informações do arquivo lido
-        """
-        leitor = LeituraDadger(diretorio)
-        r = leitor.le_arquivo(nome_arquivo)
-        return cls(r)
+        return cls.read(diretorio, nome_arquivo)
 
     def escreve_arquivo(self, diretorio: str, nome_arquivo="dadger.rv0"):
-        """
-        Realiza a escrita de um arquivo com as informações do
-        objeto :class:`Dadger`
+        self.write(diretorio, nome_arquivo)
 
-        :param diretorio: O caminho relativo ou completo para o diretório
-            onde será escrito o arquivo.
-        :type diretorio: str
-        :param nome_arquivo: Nome do arquivo a ser escrito.Tem como valor
-            default "dadger.rv0"
-        :type nome_arquivo: str, optional
+    def __registros_por_tipo(self, registro: Type[T]) -> List[T]:
         """
-        escritor = EscritaRegistros(diretorio)
-        escritor.escreve_arquivo(self._dados, nome_arquivo)
+        Obtém um gerador de blocos de um tipo, se houver algum no arquivo.
+        :param bloco: Um tipo de bloco para ser lido
+        :type bloco: T
+        :param indice: O índice do bloco a ser acessado, dentre os do tipo
+        :type indice: int
+        """
+        return [b for b in self.data.of_type(registro)]
 
-    def __obtem_registro(self, tipo: Type[T]) -> T:
+    def __obtem_registro(self, tipo: Type[T]) -> Optional[T]:
         """ """
-        for b in self._registros:
-            if isinstance(b, tipo):
-                return b
-        raise ValueError(f"Não foi encontrado um registro do tipo {tipo}")
+        r = self.__obtem_registros(tipo)
+        return r[0] if len(r) > 0 else None
+
+    def __obtem_registro_com_codigo(
+        self, tipo: Type[T], codigo: int
+    ) -> Optional[T]:
+        regs: List[Any] = self.__registros_por_tipo(tipo)
+        for r in regs:
+            if r.codigo == codigo:
+                return r
+        return None
 
     def __obtem_registro_do_estagio(
         self, tipo: Type[T], codigo: int, estagio: int
     ) -> Optional[T]:
-        regs: List[Any] = self.__obtem_registros(tipo)
+        regs: List[Any] = self.__registros_por_tipo(tipo)
         for r in regs:
             if all([r.codigo == codigo, r.estagio == estagio]):
                 return r
         return None
 
+    def __obtem_registro_do_subsistema(
+        self, tipo: Type[T], codigo: int, subsistema: int
+    ) -> Optional[T]:
+        regs: List[Any] = self.__registros_por_tipo(tipo)
+        for r in regs:
+            if all([r.codigo == codigo, r.subsistema == subsistema]):
+                return r
+        return None
+
+    def __obtem_registro_do_estagio_para_subsistema(
+        self, tipo: Type[T], estagio: int, subsistema: int
+    ) -> Optional[T]:
+        regs: List[Any] = self.__registros_por_tipo(tipo)
+        for r in regs:
+            if all([r.estagio == estagio, r.subsistema == subsistema]):
+                return r
+        return None
+
     def __obtem_registros(self, tipo: Type[T]) -> List[T]:
-        registros = []
-        for b in self._registros:
-            if isinstance(b, tipo):
-                registros.append(b)
-        return registros
+        return self.__registros_por_tipo(tipo)
+
+    def cria_registro(self, anterior: Register, registro: Register):
+        """
+        Adiciona um registro ao arquivo após um outro registro previamente
+        existente.
+
+        Este método existe para retrocompatibilidade e deve ser substituído
+        quando for suportado na classe :class:`RegisterFile`.
+        """
+        self.data.add_after(anterior, registro)
+
+    def deleta_registro(self, registro: Register):
+        """
+        Remove um registro existente no arquivo.
+
+        Este método existe para retrocompatibilidade e deve ser substituído
+        quando for suportado na classe :class:`RegisterFile`.
+        """
+        self.data.remove(registro)
 
     @property
-    def te(self) -> TE:
+    def te(self) -> Optional[TE]:
         """
         Obtém o (único) registro que define o nome do estudo no
         :class:`Dadger`
 
         :return: Um registro do tipo :class:`TE`.
         """
-        r = self.__obtem_registro(TE)
-        return r
+        return self.__obtem_registro(TE)
 
-    def sb(self, codigo: int) -> SB:
+    def sb(self, codigo: int) -> Optional[SB]:
         """
         Obtém um registro que define os subsistemas existentes
         no estudo descrito pelo :class:`Dadger`.
@@ -117,15 +298,9 @@ class Dadger(ArquivoRegistros):
         :type codigo: int
         :return: Um registro do tipo :class:`SB`
         """
-        regs: List[SB] = self.__obtem_registros(SB)
-        for r in regs:
-            if r.codigo == codigo:
-                return r
-        raise ValueError(
-            "Não foi encontrado registro SB" + f" para o código {codigo}"
-        )
+        return self.__obtem_registro_com_codigo(SB, codigo)
 
-    def uh(self, codigo: int) -> UH:
+    def uh(self, codigo: int) -> Optional[UH]:
         """
         Obtém um registro que define uma usina hidrelétrica existente
         no estudo descrito pelo :class:`Dadger`.
@@ -135,15 +310,9 @@ class Dadger(ArquivoRegistros):
         :type codigo: int
         :return: Um registro do tipo :class:`UH`
         """
-        regs: List[UH] = self.__obtem_registros(UH)
-        for r in regs:
-            if r.codigo == codigo:
-                return r
-        raise ValueError(
-            "Não foi encontrado registro UH" + f" para a UHE {codigo}"
-        )
+        return self.__obtem_registro_com_codigo(UH, codigo)
 
-    def ct(self, codigo: int, estagio: int) -> CT:
+    def ct(self, codigo: int, estagio: int) -> Optional[CT]:
         """
         Obtém um registro que define uma usina termelétrica existente
         no estudo descrito pelo :class:`Dadger`.
@@ -155,17 +324,9 @@ class Dadger(ArquivoRegistros):
         :type estagio: int
         :return: Um registro do tipo :class:`CT`
         """
-        regs: List[CT] = self.__obtem_registros(CT)
-        for r in regs:
-            if all([r.codigo == codigo, r.estagio == estagio]):
-                return r
-        raise ValueError(
-            "Não foi encontrado registro CT"
-            + f" para a UTE {codigo} no estágio"
-            + f" {estagio}."
-        )
+        return self.__obtem_registro_do_estagio(CT, codigo, estagio)
 
-    def dp(self, estagio: int, subsistema: int) -> DP:
+    def dp(self, estagio: int, subsistema: int) -> Optional[DP]:
         """
         Obtém um registro que define as durações dos patamares
         no estudo descrito pelo :class:`Dadger`.
@@ -178,24 +339,18 @@ class Dadger(ArquivoRegistros):
         :type subsistema: int
         :return: Um registro do tipo :class:`DP`
         """
-        regs: List[DP] = self.__obtem_registros(DP)
-        for r in regs:
-            if all([r.estagio == estagio, r.subsistema == subsistema]):
-                return r
-        raise ValueError(
-            "Não foi encontrado registro DP"
-            + f" para o subsistema {subsistema}"
-            + f" no estágio {estagio})"
+        return self.__obtem_registro_do_estagio_para_subsistema(
+            DP, estagio, subsistema
         )
 
     def ac(
         self,
         uhe: int,
-        modificacao: str,
+        modificacao: Any,
         mes: str = None,
         semana: int = None,
         ano: int = None,
-    ) -> AC:
+    ) -> Optional[AC]:
         """
         Obtém um registro que define modificações nos parâmetros
         das UHE em um :class:`Dadger`.
@@ -207,8 +362,10 @@ class Dadger(ArquivoRegistros):
         :return: Um registro do tipo :class:`AC`
         """
 
-        def __atende(r: AC) -> bool:
-            condicoes: List[bool] = [r.uhe == uhe, r.modificacao == modificacao]
+        def __atende(r: Dadger.AC) -> bool:
+            condicoes: List[bool] = [
+                r.uhe == uhe,
+            ]
             if mes is not None:
                 condicoes.append(r.mes == mes)
             if semana is not None:
@@ -217,17 +374,13 @@ class Dadger(ArquivoRegistros):
                 condicoes.append(r.ano == ano)
             return all(condicoes)
 
-        regs: List[AC] = self.__obtem_registros(AC)
+        regs: List[Dadger.AC] = self.__obtem_registros(modificacao)
         for r in regs:
             if __atende(r):
                 return r
-        raise ValueError(
-            "Não foi encontrado registro AC"
-            + f" para a UHE {uhe}"
-            + f" que modifique {modificacao})"
-        )
+        return None
 
-    def cd(self, numero_curva: int, subsistema: int) -> CD:
+    def cd(self, numero_curva: int, subsistema: int) -> Optional[CD]:
         """
         Obtém um registro que define as curvas de déficit
         no estudo descrito pelo :class:`Dadger`.
@@ -240,20 +393,16 @@ class Dadger(ArquivoRegistros):
         :type subsistema: int
         :return: Um registro do tipo :class:`CD`
         """
-        regs: List[CD] = self.__obtem_registros(CD)
+        regs: List[CD] = self.__registros_por_tipo(CD)
         for r in regs:
             if all(
                 [r.numero_curva == numero_curva, r.subsistema == subsistema]
             ):
                 return r
-        raise ValueError(
-            "Não foi encontrado registro CD"
-            + f" para o subsistema {subsistema}"
-            + f" na curva {numero_curva})"
-        )
+        return None
 
     @property
-    def tx(self) -> TX:
+    def tx(self) -> Optional[TX]:
         """
         Obtém o (único) registro que define a taxa de desconto
         aplicada no estudo definido no :class:`Dadger`
@@ -263,7 +412,7 @@ class Dadger(ArquivoRegistros):
         return self.__obtem_registro(TX)
 
     @property
-    def gp(self) -> GP:
+    def gp(self) -> Optional[GP]:
         """
         Obtém o (único) registro que define o gap para convergência
         considerado no estudo definido no :class:`Dadger`
@@ -273,7 +422,7 @@ class Dadger(ArquivoRegistros):
         return self.__obtem_registro(GP)
 
     @property
-    def ni(self) -> NI:
+    def ni(self) -> Optional[NI]:
         """
         Obtém o (único) registro que define o número máximo de iterações
         do DECOMP no estudo definido no :class:`Dadger`
@@ -283,7 +432,7 @@ class Dadger(ArquivoRegistros):
         return self.__obtem_registro(NI)
 
     @property
-    def dt(self) -> DT:
+    def dt(self) -> Optional[DT]:
         """
         Obtém o (único) registro que define a data de referência do
         estudo definido no :class:`Dadger`
@@ -292,7 +441,7 @@ class Dadger(ArquivoRegistros):
         """
         return self.__obtem_registro(DT)
 
-    def re(self, codigo: int) -> RE:
+    def re(self, codigo: int) -> Optional[RE]:
         """
         Obtém um registro que cadastra uma restrição elétrica existente
         no estudo descrito pelo :class:`Dadger`.
@@ -302,15 +451,9 @@ class Dadger(ArquivoRegistros):
         :type codigo: int
         :return: Um registro do tipo :class:`RE`
         """
-        regs: List[RE] = self.__obtem_registros(RE)
-        for r in regs:
-            if r.codigo == codigo:
-                return r
-        raise ValueError(
-            "Não foi encontrado registro RE" + f" com código {codigo}"
-        )
+        return self.__obtem_registro_com_codigo(RE, codigo)
 
-    def lu(self, codigo: int, estagio: int) -> LU:
+    def lu(self, codigo: int, estagio: int) -> Optional[LU]:
         """
         Obtém um registro que especifica os limites inferiores e
         superiores por patamar de uma restrição elétrica existente
@@ -353,47 +496,43 @@ class Dadger(ArquivoRegistros):
 
         """
 
-        def cria_registro(modelo: LU, estagio_final: int) -> LU:
-            copia = deepcopy(modelo)
-            copia.estagio = estagio
-            # Procura pelo registro do próximo estágio
-            proximo = None
-            for est in range(estagio, estagio_final + 1):
-                r = self.__obtem_registro_do_estagio(LU, codigo, est)
-                if r is not None:
-                    proximo = r
-                    break
-            if proximo is not None:
-                delta = (proximo._ordem - modelo._ordem) / 2.0
-            else:
-                delta = 0.1
-            copia._ordem += delta
-            self._registros.append(copia)
-            return copia
+        def cria_registro() -> Optional[LU]:
+            re = self.__obtem_registro_com_codigo(RE, codigo)
+            if re is None:
+                return re
+            ei = re.estagio_inicial
+            ef = re.estagio_final
+            ultimo_registro = None
+            if ei is not None and ef is not None:
+                for e in range(ei, ef + 1):
+                    registro_estagio = self.__obtem_registro_do_estagio(
+                        LU, codigo, e
+                    )
+                    if registro_estagio is not None:
+                        ultimo_registro = registro_estagio
+            if isinstance(ultimo_registro, LU):
+                novo_registro = LU(
+                    state=ComponentState.READ_SUCCESS,
+                    data=[None] * len(ultimo_registro.data),
+                )
+                novo_registro.codigo = ultimo_registro.codigo
+                novo_registro.limites_inferiores = (
+                    ultimo_registro.limites_inferiores
+                )
+                novo_registro.limites_superiores = (
+                    ultimo_registro.limites_superiores
+                )
+                novo_registro.estagio = estagio
+                self.data.add_after(ultimo_registro, novo_registro)
+                return novo_registro
+            return None
 
-        # Obtém o registro RE associado
-        re = self.re(codigo)
-        # Confere se o estágio pedido está no intervalo
-        # do registro RE
-        if not (re.estagio_inicial <= estagio <= re.estagio_final):
-            raise ValueError(
-                f"Estágio {estagio} fora dos limites" + " do registro RE"
-            )
-        # Tenta obter um registro já existente
-        reg = self.__obtem_registro_do_estagio(LU, codigo, estagio)
-        if reg is not None:
-            return reg
-        # Se não conseguir, cria mais um registro, idêntico ao do
-        # último estágio existente, e retorna.
-        for est in range(re.estagio_inicial, estagio):
-            r = self.__obtem_registro_do_estagio(LU, codigo, est)
-            if r is not None:
-                reg = r
-        if reg is None:
-            raise ValueError("Registro não encontrado")
-        return cria_registro(reg, re.estagio_final)
+        lu = self.__obtem_registro_do_estagio(LU, codigo, estagio)
+        if lu is None:
+            lu = cria_registro()
+        return lu
 
-    def vi(self, uhe: int) -> VI:
+    def vi(self, uhe: int) -> Optional[VI]:
         """
         Obtém um registro que especifica os tempos de viagem da
         água em uma UHE existente no no estudo descrito
@@ -407,11 +546,9 @@ class Dadger(ArquivoRegistros):
         for r in regs:
             if r.uhe == uhe:
                 return r
-        raise ValueError(
-            "Não foi encontrado registro VI" + f" para a UHE {uhe}"
-        )
+        return None
 
-    def ir(self, tipo: str) -> IR:
+    def ir(self, tipo: str) -> Optional[IR]:
         """
         Obtém um registro que especifica os relatórios de saída
         a serem produzidos pelo DECOMP após a execução do estudo
@@ -426,11 +563,9 @@ class Dadger(ArquivoRegistros):
         for r in regs:
             if r.tipo == tipo:
                 return r
-        raise ValueError(
-            "Não foi encontrado registro IR" + f" com mnemônico {tipo}"
-        )
+        return None
 
-    def rt(self, mnemonico: str) -> RT:
+    def rt(self, mnemonico: str) -> Optional[RT]:
         """
         Obtém um registro que especifica uma retirada de restrição
         de soleira de vertedouro ou canal de desvio.
@@ -444,11 +579,9 @@ class Dadger(ArquivoRegistros):
         for r in regs:
             if r.restricao == mnemonico:
                 return r
-        raise ValueError(
-            "Não foi encontrado registro RT" + f" com mnemônico {mnemonico}"
-        )
+        return None
 
-    def fc(self, tipo: str) -> FC:
+    def fc(self, tipo: str) -> Optional[FC]:
         """
         Obtém um registro que especifica os caminhos para os
         arquivos com a FCF do NEWAVE.
@@ -462,11 +595,9 @@ class Dadger(ArquivoRegistros):
         for r in regs:
             if r.tipo == tipo:
                 return r
-        raise ValueError(
-            "Não foi encontrado registro FC" + f" com mnemônico {tipo}"
-        )
+        return None
 
-    def ti(self, codigo: int) -> TI:
+    def ti(self, codigo: int) -> Optional[TI]:
         """
         Obtém um registro que especifica as taxas de irrigação
         por posto (UHE) existente no estudo especificado no :class:`Dadger`
@@ -480,11 +611,9 @@ class Dadger(ArquivoRegistros):
         for r in regs:
             if r.codigo == codigo:
                 return r
-        raise ValueError(
-            "Não foi encontrado registro TI" + f" para a UHE {codigo}"
-        )
+        return None
 
-    def fp(self, codigo: int, estagio: int) -> FP:
+    def fp(self, codigo: int, estagio: int) -> Optional[FP]:
         """
         Obtém um registro que especifica as taxas de irrigação
         por posto (UHE) existente no estudo especificado no :class:`Dadger`
@@ -500,12 +629,9 @@ class Dadger(ArquivoRegistros):
         if r is not None:
             return r
         else:
-            raise ValueError(
-                "Registro não encontrado registro FP"
-                + f"para a UHE {codigo} no estágio {estagio}"
-            )
+            return None
 
-    def rq(self, ree: int) -> RQ:
+    def rq(self, ree: int) -> Optional[RQ]:
         """
         Obtém um registro que especifica as vazões mínimas históricas
         por REE existentes no estudo especificado no :class:`Dadger`
@@ -518,11 +644,9 @@ class Dadger(ArquivoRegistros):
         for r in regs:
             if r.ree == ree:
                 return r
-        raise ValueError(
-            "Não foi encontrado registro RQ" + f" para o REE {ree}"
-        )
+        return None
 
-    def ve(self, codigo: int) -> VE:
+    def ve(self, codigo: int) -> Optional[VE]:
         """
         Obtém um registro que especifica os volumes de espera
         por posto (UHE) existente no estudo especificado no :class:`Dadger`
@@ -535,11 +659,9 @@ class Dadger(ArquivoRegistros):
         for r in regs:
             if r.codigo == codigo:
                 return r
-        raise ValueError(
-            "Não foi encontrado registro VE" + f" para a UHE {codigo}"
-        )
+        return None
 
-    def hv(self, codigo: int) -> HV:
+    def hv(self, codigo: int) -> Optional[HV]:
         """
         Obtém um registro que cadastra uma restrição de volume mínimo
         armazenado existente no estudo descrito pelo :class:`Dadger`.
@@ -553,11 +675,9 @@ class Dadger(ArquivoRegistros):
         for r in regs:
             if r.codigo == codigo:
                 return r
-        raise ValueError(
-            "Não foi encontrado registro HV" + f" para UHE {codigo}"
-        )
+        return None
 
-    def lv(self, codigo: int, estagio: int) -> LV:
+    def lv(self, codigo: int, estagio: int) -> Optional[LV]:
         """
         Obtém um registro que especifica os limites inferior e
         superior de uma restrição de volume mínimo existente
@@ -600,47 +720,39 @@ class Dadger(ArquivoRegistros):
 
         """
 
-        def cria_registro(modelo: LV, estagio_final: int) -> LV:
-            copia = deepcopy(modelo)
-            copia.estagio = estagio
-            # Procura pelo registro do próximo estágio
-            proximo = None
-            for est in range(estagio, estagio_final + 1):
-                r = self.__obtem_registro_do_estagio(LV, codigo, est)
-                if r is not None:
-                    proximo = r
-                    break
-            if proximo is not None:
-                delta = (proximo._ordem - modelo._ordem) / 2.0
-            else:
-                delta = 0.1
-            copia._ordem += delta
-            self._registros.append(copia)
-            return copia
+        def cria_registro() -> Optional[LV]:
+            hv = self.__obtem_registro_com_codigo(HV, codigo)
+            if hv is None:
+                return hv
+            ei = hv.estagio_inicial
+            ef = hv.estagio_final
+            ultimo_registro = None
+            if ei is not None and ef is not None:
+                for e in range(ei, ef + 1):
+                    registro_estagio = self.__obtem_registro_do_estagio(
+                        LV, codigo, e
+                    )
+                    if registro_estagio is not None:
+                        ultimo_registro = registro_estagio
+            if isinstance(ultimo_registro, LV):
+                novo_registro = LV(
+                    state=ComponentState.READ_SUCCESS,
+                    data=[None] * len(ultimo_registro.data),
+                )
+                novo_registro.codigo = codigo
+                novo_registro.limite_inferior = ultimo_registro.limite_inferior
+                novo_registro.limite_superior = ultimo_registro.limite_superior
+                novo_registro.estagio = estagio
+                self.data.add_after(ultimo_registro, novo_registro)
+                return novo_registro
+            return None
 
-        # Obtém o registro HV associado
-        hv = self.hv(codigo)
-        # Confere se o estágio pedido está no intervalo
-        # do registro HV
-        if not (hv.estagio_inicial <= estagio <= hv.estagio_final):
-            raise ValueError(
-                f"Estágio {estagio} fora dos limites" + " do registro HV"
-            )
-        # Tenta obter um registro já existente
-        reg = self.__obtem_registro_do_estagio(LV, codigo, estagio)
-        if reg is not None:
-            return reg
-        # Se não conseguir, cria mais um registro, idêntico ao do
-        # último estágio existente, e retorna.
-        for est in range(hv.estagio_inicial, estagio):
-            r = self.__obtem_registro_do_estagio(LV, codigo, est)
-            if r is not None:
-                reg = r
-        if reg is None:
-            raise ValueError("Registro não encontrado")
-        return cria_registro(reg, hv.estagio_final)
+        lv = self.__obtem_registro_do_estagio(LV, codigo, estagio)
+        if lv is None:
+            lv = cria_registro()
+        return lv
 
-    def hq(self, codigo: int) -> HQ:
+    def hq(self, codigo: int) -> Optional[HQ]:
         """
         Obtém um registro que cadastra uma restrição de vazão
         existente no estudo descrito pelo :class:`Dadger`.
@@ -654,11 +766,9 @@ class Dadger(ArquivoRegistros):
         for r in regs:
             if r.codigo == codigo:
                 return r
-        raise ValueError(
-            "Não foi encontrado registro HQ" + f" com o código {codigo}"
-        )
+        return None
 
-    def lq(self, codigo: int, estagio: int) -> LQ:
+    def lq(self, codigo: int, estagio: int) -> Optional[LQ]:
         """
         Obtém um registro que especifica os limites inferiores e
         superiores por patamar de uma restrição de vazão existente
@@ -701,47 +811,43 @@ class Dadger(ArquivoRegistros):
 
         """
 
-        def cria_registro(modelo: LQ, estagio_final: int) -> LQ:
-            copia = deepcopy(modelo)
-            copia.estagio = estagio
-            # Procura pelo registro do próximo estágio
-            proximo = None
-            for est in range(estagio, estagio_final + 1):
-                r = self.__obtem_registro_do_estagio(LQ, codigo, est)
-                if r is not None:
-                    proximo = r
-                    break
-            if proximo is not None:
-                delta = (proximo._ordem - modelo._ordem) / 2.0
-            else:
-                delta = 0.1
-            copia._ordem += delta
-            self._registros.append(copia)
-            return copia
+        def cria_registro() -> Optional[LQ]:
+            hq = self.__obtem_registro_com_codigo(HQ, codigo)
+            if hq is None:
+                return hq
+            ei = hq.estagio_inicial
+            ef = hq.estagio_final
+            ultimo_registro = None
+            if ei is not None and ef is not None:
+                for e in range(ei, ef + 1):
+                    registro_estagio = self.__obtem_registro_do_estagio(
+                        LQ, codigo, e
+                    )
+                    if registro_estagio is not None:
+                        ultimo_registro = registro_estagio
+            if isinstance(ultimo_registro, LQ):
+                novo_registro = LQ(
+                    state=ComponentState.READ_SUCCESS,
+                    data=[None] * len(ultimo_registro.data),
+                )
+                novo_registro.codigo = codigo
+                novo_registro.limites_superiores = (
+                    ultimo_registro.limites_superiores
+                )
+                novo_registro.limites_inferiores = (
+                    ultimo_registro.limites_inferiores
+                )
+                novo_registro.estagio = estagio
+                self.data.add_after(ultimo_registro, novo_registro)
+                return novo_registro
+            return None
 
-        # Obtém o registro HQ associado
-        hq = self.hq(codigo)
-        # Confere se o estágio pedido está no intervalo
-        # do registro HQ
-        if not (hq.estagio_inicial <= estagio <= hq.estagio_final):
-            raise ValueError(
-                f"Estágio {estagio} fora dos limites" + " do registro HQ"
-            )
-        # Tenta obter um registro já existente
-        reg = self.__obtem_registro_do_estagio(LQ, codigo, estagio)
-        if reg is not None:
-            return reg
-        # Se não conseguir, cria mais um registro, idêntico ao do
-        # último estágio existente, e retorna.
-        for est in range(hq.estagio_inicial, estagio):
-            r = self.__obtem_registro_do_estagio(LQ, codigo, est)
-            if r is not None:
-                reg = r
-        if reg is None:
-            raise ValueError("Registro não encontrado")
-        return cria_registro(reg, hq.estagio_final)
+        lq = self.__obtem_registro_do_estagio(LQ, codigo, estagio)
+        if lq is None:
+            lq = cria_registro()
+        return lq
 
-    def he(self, codigo: int, estagio: int) -> HE:
+    def he(self, codigo: int, estagio: int) -> Optional[HE]:
         """
         Obtém um registro que cadastra uma restrição de energia
         armazenada existente no estudo descrito pelo :class:`Dadger`.
@@ -758,26 +864,39 @@ class Dadger(ArquivoRegistros):
         if r is not None:
             return r
         else:
-            raise ValueError("Registro não encontrado")
+            return None
+
+    def cm(self, codigo: int) -> Optional[CM]:
+        """
+        Obtém um registro que cadastra os coeficientes das restrições
+        de energia armazenada.
+
+        :param codigo: Índice do código que especifica o registro
+        :type codigo: int
+        :return: Um registro do tipo :class:`CM`
+        """
+        regs: List[CM] = self.__obtem_registros(CM)
+        for r in regs:
+            if r.codigo == codigo:
+                return r
+        return None
 
     @property
-    def ev(self) -> EV:
+    def ev(self) -> Optional[EV]:
         """
         Obtém o (único) registro que define a evaporação
         :class:`Dadger`
 
         :return: Um registro do tipo :class:`EV`.
         """
-        r = self.__obtem_registro(EV)
-        return r
+        return self.__obtem_registro(EV)
 
     @property
-    def fj(self) -> FJ:
+    def fj(self) -> Optional[FJ]:
         """
         Obtém o (único) registro que define o arquivo `polinjus`
         :class:`Dadger`
 
         :return: Um registro do tipo :class:`FJ`.
         """
-        r = self.__obtem_registro(FJ)
-        return r
+        return self.__obtem_registro(FJ)
